package Hawat::Controller::Secure;
use strict;
use warnings;
use Moose;
use namespace::autoclean;

use Scalar::Util 'blessed';
use Data::Dumper;

use Mentat::Core qw(:c_hawat);

BEGIN { extends 'Hawat::Controller::Base' }

=head1 NAME

Hawat::Controller::Secure - Base class for all application controllers with restricted actions

=head1 DESCRIPTION

Base class for all application controllers, which provide actions with restricted access.

=head1 METHODS

=head2 begin

Implementation of "auto-login" feature

=cut

sub begin : Private {
    my ( $self, $c ) = @_;

    # User authentication may be turned off via configuration file
    if ($c->cfg('authentication')) {

        # If a user doesn't exist, force login
        unless ($c->user_exists) {
            $c->log->debug('***Secure::begin - User not found, redirecting to /login');

            # Redirect the user to the 'login' page
            $c->flash->{redirect_after_login} = ''.$c->req->uri;
            $c->response->redirect($c->uri_for('/login'));
            $c->detach;
        }
        # Otherwise perform base role check
        elsif (defined $c->cfg('acl')->{base_role}) {
            $c->log->debug('***Secure::begin - Checking base role');

            unless ($c->check_user_roles($c->cfg('acl')->{base_role})) {
                # Dump a log message to the development server debug output
                $c->log->debug('***Secure::begin - User not activated, detaching to /activation');

                # Redirect the user to the 'activation' page
                $c->detach($c->uri_for('/activation'));
                # FIXME: It would be better to redirect for activation page, but following causes
                # the redirect loop, fix it
                #$c->response->redirect('/activation');
                #$c->detach;
            }
        }

    }
    $c->session->{sequence}++;
}

=head1 AUTHOR

 Jan Mach
 Cesnet, z.s.p.o
 jan.mach@cesnet.cz
 http://www.cesnet.cz

=head1 LICENSE

This library is free software. You can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

__PACKAGE__->meta->make_immutable;

1;
