package Hawat::Model::Paginator;
use strict;
use warnings;
use base 'Catalyst::Model::Adaptor';

#COMPONENT will be called only once when Catalyst starts so you can merge the config and initialize your model internals.
#ACCEPT_CONTEXT will be called when you use $c->model([instance])->method();
#http://search.cpan.org/~jjnapiork/Catalyst-Runtime-5.90018/lib/Catalyst/Component.pm

#-- Perl core modules ---------------------------------------------------------#
use Carp;
use Scalar::Util 'blessed';

use POSIX;

__PACKAGE__->config(
    class       => 'Hawat::Model::Paginator',
    constructor => 'new',
);

# Calculate database skip based on the limit of records per page and current page number
sub calculate_skip($$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($limit, $page) = @_;
    confess "Expected limit and page number as arguments" unless ($limit and $page);

    return ($page - 1) * $limit;
}

# Calculate the data structure for paginator web object
sub calculate($$$$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($limit, $page, $count, $count_all) = @_;
    confess "Expected limit and page number as arguments" unless ($limit and $page);

    $count = 0 unless $count; $count_all = 0 unless $count_all;

    return {
            'limit'         => $limit,
            'count'         => $count,
            'count_all'     => $count_all,
            'page'          => $page,
            'page_first'    => 1,
            'page_last'     => ceil($count_all/$limit),
            'page_previous' => $page - 1,
            'page_next'     => $page + 1,
            'imin'          => ($count)?((($page - 1) * $limit) + 1):0,
            'imax'          => (($page - 1) * $limit) + $count,
        };
}

1;
